/*
 *
 * Edscott Wilson Garcia 2001-5
 * Copyright (C) 1998 Rasca, Berlin
 * EMail: thron@gmx.de
 *
 * Olivier Fourdan (fourdan@xfce.org)
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <glib.h>
#include <gmodule.h>

#include "primary.h"

#include "uri.h"

G_MODULE_EXPORT
const gchar *
uri_remove_file_prefix(gchar *path)
{
    static gchar *url_he=NULL;
    if (strncmp(path,"file:/",strlen("file:/"))==0 && !strstr(path,"file://")) 
    {
	/* assume local */
	const gchar *f= "file:";
	memmove(path, path + strlen(f), strlen(path + strlen("file:"))+1);
	TRACE("case file:/");
	return NULL;
    }
    if (strncmp(path,"file:///",strlen("file:///"))==0) 
    {
	/* assume local again */
	const gchar *f= "file://";
	memmove(path, path + strlen(f), strlen(path + strlen(f))+1);
	TRACE("case file:///");
	return NULL;
    }
    
    if (strncmp(path,"file://",strlen("file://"))==0) 
    {
	/* local? */
	const gchar *f= "file:/";
	if (g_file_test(path+strlen(f),G_FILE_TEST_EXISTS)){
	    /* local it is */	
	    TRACE("case local file://");
	    memmove(path, path + strlen(f), strlen(path + strlen(f))+1);
	    return NULL;
	}	
	TRACE("case remote file://");
	memmove(path, path + strlen("file://"), strlen(path + strlen("file://"))+1);    
	g_free(url_he);
	url_he=g_strdup(path);
	if (strchr(url_he,'/')) *strchr(url_he,'/') = 0; 
	memmove(path, path + strlen(url_he), strlen(path + strlen(url_he))+1); 	
	return url_he;
    }
	TRACE("not file:");
    return (NULL);
}

G_MODULE_EXPORT
void uri_remove_file_prefix_from_list(GList * list, const gchar *he, const gchar *me)
{ 
    while(list)  {
	gchar *url = (gchar *) list->data;
	if (strncmp(url,"file:",strlen("file:"))==0) {
	    const gchar *url_he;
	    TRACE("url=%s",url);
	    url_he=uri_remove_file_prefix(url);
	    TRACE("removed url=%s",url);
	    
	    if (strcmp(me,he)) {
		/* remote request */
		gchar *fixed_url;
		if (url_he){
		    fixed_url = g_strdup_printf("%s:%s",url_he,url);
		} else {
		    fixed_url = g_strdup_printf("%s:%s",he,url);
		}
		g_free(list->data);
		list->data=fixed_url;
	    }
	    TRACE("fixed url=%s",url);
	}
	list = list->next;
    }
    return ;
}


G_MODULE_EXPORT
int uri_parse_list(const gchar *text, GList ** list)
{
    gchar *q,*p,*linesep;
    
    q=p=g_strdup(text);

    if (strstr(p,"\r\n")) linesep="\r\n";
    else if (strstr(p,"\n")) linesep="\n";
    else if (strstr(p,"\r")) linesep="\r";
    else return FALSE;
    
    for (p=strtok(p,linesep); p; p=strtok(NULL,linesep)){
	gchar *t=g_strdup(p);
	*list=g_list_append(*list,t);
    }
    g_free(q);
    return TRUE;
}

G_MODULE_EXPORT
GList *uri_free_list(GList * list)
{
    GList *t;
    for (t=list; t; t=t->next) g_free(t->data);
    g_list_free(list);
    return NULL;
}


